"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const types_1 = require("../plugins/types");
/**
 * Watches all source files for changes and de-dupes changes that occur within the specified delay window.
 * @internal
 */
// tslint:disable-next-line: no-async-without-await
async function* watchAllSources(plugins, delay, context) {
    let watchers = plugins.filter(types_1.hasWatch);
    let fileGenerators = watchers.map((watcher) => watcher.watch(context));
    let batchedFileChanges = utils_1.debounceIterable(utils_1.joinIterables(...fileGenerators), delay);
    for await (let changes of batchedFileChanges) {
        let groupedChanges = groupByFile(changes);
        yield dedupeFileChanges(groupedChanges);
    }
}
exports.watchAllSources = watchAllSources;
/**
 * Groups the given file changes by file path, so we can detect any duplicates
 */
function groupByFile(changes) {
    let groupedChanges = new Map();
    for (let change of changes) {
        let key = change.file.path;
        let fileChanges = groupedChanges.get(key) || [];
        fileChanges.push(change);
        groupedChanges.set(key, fileChanges);
    }
    return groupedChanges;
}
/**
 * De-duplicates file changes. For example, if a file was deleted and then re-created, it will
 * be de-duped as a single modification.
 */
function dedupeFileChanges(groupedChanges) {
    let changedFiles = [];
    for (let changes of groupedChanges.values()) {
        if (changes.length > 1) {
            // Sort the changes by timestamp, and get the first and last change
            changes.sort(byTimestamp);
            let first = changes[0];
            let last = changes[changes.length - 1];
            switch (last.file.change) {
                case "created" /* Created */:
                    if (first.file.change !== "created" /* Created */) {
                        // The file was re-created after being deleted/modified,
                        // so this is actually just a modification
                        last.file.change = "modified" /* Modified */;
                    }
                    break;
                case "modified" /* Modified */:
                    if (first.file.change === "created" /* Created */) {
                        // The file was created and then modified, so this is actually a creation
                        last.file.change = "created" /* Created */;
                    }
                    break;
                case "deleted" /* Deleted */:
                default:
                    // The previous changes don't matter, because the file was ultimately deleted
                    break;
            }
            // Always use the latest contents we have
            last.file.contents = getLatestContents(changes);
            changedFiles.push(last.file);
        }
        else {
            // Nothig to de-dupe
            changedFiles.push(changes[0].file);
        }
    }
    return changedFiles;
}
/**
 * Sorts an array of `CreateFileInfo` objects by their modified time
 */
function byTimestamp(a, b) {
    return a.file.modifiedAt.valueOf() - b.file.modifiedAt.valueOf();
}
/**
 * Returns the latest contents from a sorted list of `CreateFileInfo` objects.
 */
function getLatestContents(changes) {
    for (let i = changes.length - 1; i >= 0; i--) {
        if (changes[i].hasContents) {
            return changes[i].file.contents;
        }
    }
    return Buffer.alloc(0);
}
//# sourceMappingURL=watch.js.map